using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Image_Io(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            /* Sample: Image_Io
             * This sample will show you how to:
             * - Load a JPG file through HPS.Image and check for Image IO exceptions
             * - Obtain an ImageKit
             * - Obtain a TextKit
             * - Convert an image between formats
             * - Output an image to a file
             * - Import an image from a file
             * - Insert text
             * - Insert an Image
             * 
             * Read more about the topics covered in this sample in our Programming Guide 
             * - section 8.3:	Loading 2D Images */

            HPS.SegmentKey results_segment = modelKey.Subsegment();

            HPS.TextKit textKit = new HPS.TextKit();
            textKit.SetSize(0.02f, Text.SizeUnits.SubscreenRelative);
            HPS.Point[] point_array = { new HPS.Point(-0.95f, -0.70f, 10), new HPS.Point(0.95f, -0.70f, 10), new HPS.Point(0, -1, 10) };
            textKit.SetRegion(point_array, Text.RegionAlignment.Center, Text.RegionFitting.Left, false, false, true);
            textKit.SetBackground(true, "rounded box");
            textKit.SetAlignment(HPS.Text.Alignment.BottomLeft);
            results_segment.GetMaterialMappingControl().SetFaceColor(new RGBAColor(1, 1, 1));
            results_segment.GetMaterialMappingControl().SetEdgeColor(new RGBAColor(0, 0, 0));
            results_segment.GetVisibilityControl().SetEdges(true);

            /* This next block of code shows you how to load an HSF file
             * 1. Create an ImportOptionsKit and set the format of the image to be read in
             * 2. Use the Image.File.Import method to load the desired image, with the options you specified
             * 3. An ImageKit describing the inserted image is returned by Image.File.Import
             * 4. Use a try-catch block to capture any Image IO exceptions */

            HPS.ImageKit jpgImageKit = new HPS.ImageKit();
            uint imageWidth, imageHeight;
            try
            {
                string file_in = INPUT_FOLDER + "/sprocket_image.jpg";
                HPS.Image.ImportOptionsKit importOptionsKit = new HPS.Image.ImportOptionsKit();
                importOptionsKit.SetFormat(HPS.Image.Format.Jpeg);
                jpgImageKit = HPS.Image.File.Import(file_in, importOptionsKit);
            }
            catch (HPS.IOException)
            {
                string error_message;
                error_message = "Image Import Unsuccessful";
                textKit.SetText(error_message);
                modelKey.InsertText(textKit);
                view.Update();
            }

            /* The next block of code shows how to convert between image formats and write an image to a file
	         * 1. Create a new ImageKit object and use its Convert method to change formats
	         * 2. The first argument of Convert is the source ImageKit
	         * 3. The second argument of Convert is the desired output format
	         * 4. Once the new ImageKit is obtained through Convert, obtain its data using the ImageKit.ShowData() method
	         * 5. Write the data out to a file in binary mode */
            if (!jpgImageKit.Empty())
            {
                bool exportSuccessful = true;
                HPS.ImageKit pngImageKit = new HPS.ImageKit();
                byte[] imageData;

                jpgImageKit.ShowSize(out imageWidth, out imageHeight);
                pngImageKit.SetFormat(HPS.Image.Format.Png).SetSize(imageWidth, imageHeight);
                pngImageKit.Convert(jpgImageKit, HPS.Image.Format.Png);
                pngImageKit.ShowData(out imageData);

                string file_out = OUTPUT_FOLDER + "/image_io.png";


                try
                {
                    using (System.IO.BinaryWriter writer = new System.IO.BinaryWriter(System.IO.File.OpenWrite(file_out)))
                    {
                        writer.Write(imageData);
                        writer.Close();
                    }
                }
                catch (System.Exception)
                {
                    exportSuccessful = false;
                }

                if (!exportSuccessful)
                {
                    string error_message;
                    error_message = "Image Export Unsuccessful";
                    textKit.SetText(error_message);
                    modelKey.InsertText(textKit);
                    view.Update();
                }
                else
                {
                    /* The next block of code shows you how to delete geometry from a segment and how to insert text using a previously defined TextKit
			         * 1. The Flush method can be used to delete different elements from a segment
			         * 2. The first argument of Flush specifies what to delete
			         * 3. The second argument specifies which part of the scene graph will be affected by this operation
			         * 4. To insert text, specify the text you want to display and then insert the TextKit to the SegmentKey */

                    string success_message = "Import and Export completed successfully.\n" +
                        "You can find the PNG image here:\n" +
                        OUTPUT_FOLDER + "/image_io.png";

                    int index = success_message.IndexOf(@"/");
                    success_message = success_message.Substring(0, index) + "\n" + success_message.Substring(index, success_message.Length - index);

                    textKit.SetText(success_message);
                    results_segment.InsertText(textKit);

                    #region Image_View

                    /* The next block of code shows how to import an image, convert it to a texture, and place it in the scene using a Shell
                     * 1. Define a ShellKit to serve as a display for the image
                     * 2. Create a rectangle with the Visualize shell primitive
                     * 2. Define a PortfolioKey to hold a texture
                     * 3. Define an ImageKit to hold the imported image
                     * 4. Define an import options kit to declare the file format(png, jpeg, gif etc.)
                     * 5. Import the image into your ImageKit using the file's path as the first parameter and the optionskit as the second parameter 
                     * 6. Use a try-catch block to capture any Stream IO exceptions */

                    float heightRatio;
                    float widthRatio;
                    float heightMargin = 0.05f;
                    float widthMargin = 0.05f;
                    if (imageHeight >= imageWidth)
                    {
                        heightRatio = 1.0f;
                        widthRatio = (float)imageWidth / imageHeight;
                        widthMargin = widthMargin * widthRatio;
                        heightRatio -= heightMargin;
                        widthRatio -= widthMargin;
                    }
                    else
                    {
                        heightRatio = (float)imageHeight / imageWidth;
                        widthRatio = 1.0f;
                        heightMargin = heightMargin * heightRatio;
                        heightRatio -= heightMargin;
                        widthRatio -= widthMargin;
                    }

                    ShellKit myShellKit = new ShellKit();
                    Point[] points = { new Point(widthRatio, -heightRatio, 0), new Point(-widthRatio, -heightRatio, 0), new Point(-widthRatio, heightRatio, 0), new Point(widthRatio, heightRatio, 0) };
                    int[] faceList = { 4, 0, 1, 2, 3 };

                    ulong[] vertices_map = new ulong[4];
                    vertices_map[0] = 0;
                    vertices_map[1] = 1;
                    vertices_map[2] = 2;
                    vertices_map[3] = 3;

                    float[] vparams = new float[8];
                    vparams[0] = 0; vparams[1] = 0;   // [0, 0]
                    vparams[2] = 1; vparams[3] = 0;   // [1, 0]
                    vparams[4] = 1; vparams[5] = 1;   // [1, 1]
                    vparams[6] = 0; vparams[7] = 1;   // [0, 1]

                    myShellKit.SetPoints(points);
                    myShellKit.SetFacelist(faceList);
                    myShellKit.SetVertexParametersByList(vertices_map, vparams, 2);

                    SegmentKey imageSegment = modelKey.Subsegment("Image");
                    imageSegment.InsertShell(myShellKit);
                    imageSegment.GetVisibilityControl().SetFaceLights(false);
                    HPS.PortfolioKey portfolio_key = HPS.Database.CreatePortfolio();
                    HPS.ImageKit image_kit = new HPS.ImageKit();

                    try
                    {
                        string filename = OUTPUT_FOLDER + "/image_io.png";

                        HPS.Image.ImportOptionsKit importOptionsKit = new HPS.Image.ImportOptionsKit();
                        importOptionsKit.SetFormat(HPS.Image.Format.Png);
                        image_kit = HPS.Image.File.Import(filename, importOptionsKit);
                    }
                    catch (HPS.IOException)
                    {
                        /* catch Stream I/O Errors here */
                        exportSuccessful = false;
                    }

                    /* The next block of code will show how to convert an image to a texture in the Visualize context and place it in your scene
                     * 1. Create an ImageDefinition within the portfolio key and a new textureOptionsKit
                     * 2. Set the ParameterizationSource as UV. See section 5.3 for more details 
                     * 3. Define the texture within the portfolio key 
                     * 4. Assign the portfolio to the segment
                     * 5. Set the model key face texture the new texture within the portfolio using it's earlier defined name to find it */


                    HPS.ImageDefinition imageDefinition = portfolio_key.DefineImage("imported_image", image_kit);
                    HPS.TextureOptionsKit textureOptionsKit = new HPS.TextureOptionsKit();

                    textureOptionsKit.SetParameterizationSource(HPS.Material.Texture.Parameterization.UV);
                    portfolio_key.DefineTexture("my_texture", imageDefinition, textureOptionsKit);

                    modelKey.GetPortfolioControl().Push(portfolio_key);
                    modelKey.GetMaterialMappingControl().SetFaceTexture("my_texture");

                    view.Update();
                    #endregion
                }

            }

        }
    }
}
