using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Image_Output(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            /* Sample: Image_Output
	         * This sample will show you how to:
	         * - Load an HSF file through Stream and check for Stream IO exceptions
	         * - Create an Offscreen Window
	         * - Render to an offscreen window
	         * - Save a snapshot of the screen to a file
	         * 
	         * Read more about the topics covered in this sample in our Programming Guide 
	         * - section 6.2:	Offscreen Rendering */

            HPS.Stream.ImportNotifier streamNotifier;
            HPS.SegmentKey geometry = modelKey.Subsegment();

            /* This next block of code shows you how to load an HSF file
	         * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
	         * 2. Use the Stream.File.Import method to load the desired file, with the options you specified
	         * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
	         * 4. Use a try-catch block to capture any Stream IO exceptions */
            try
            {
                HPS.Stream.ImportOptionsKit importOptionsKit = new HPS.Stream.ImportOptionsKit();
                importOptionsKit.SetSegment(geometry);
                string filename = INPUT_FOLDER + "/conrod.hsf";

                streamNotifier = HPS.Stream.File.Import(filename, importOptionsKit);
                streamNotifier.Wait();
            }
            catch (HPS.IOException)
            {
                /* catch Stream I/O Errors here */
            }

            geometry.GetModellingMatrixControl().Rotate(-90, 0, 0);

            /* This block of code shows how to create an off-screen window and how to render to it  
	         * 1. Create an OffScreenWindowOptionsKit object, and set all the options related to the off-screen window
	         *		such as the driver to use, or whether it will use anti-aliasing
	         * 2. To have an off-screen window with the same dimensions of the current on-screen window the size in pixels
	         *		of the on-screen window can be obtained from the WindowInfoControl
	         * 3. Crate an OffscreenWindow from the Database. Pass the width, height and options to the constructor
	         * 4. Set a camera, visibility and any other attribute as necessary on the off-screen window. In order to obtain an exact
	         *		snapshot of the on-screen window, these attributes should be the same between on- and off-screen windows
	         * 5. Include a segment containing geometry into the off-screen window. Only geometry which is either in the included segment 
	         *		or under it in the scene graph will be rendered to the off-screen window
	         * 6. Update the off-screen window to start rendering to it. Since this operation occurs on a separate thread, it is 
	         *		possible to wait for it to complete by using the UpdateNotifier.Wait() method */
            HPS.CameraKit cameraKit;
            HPS.VisibilityKit visibilityKit;
            view.FitWorld();
            modelKey.ShowCamera(out cameraKit);
            modelKey.ShowVisibility(out visibilityKit);

            HPS.OffScreenWindowOptionsKit oswok = new HPS.OffScreenWindowOptionsKit();
            oswok.SetDriver(Window.Driver.OpenGL2).SetAntiAliasCapable(true);

            uint width, height;
            canvas.GetWindowKey().GetWindowInfoControl().ShowPhysicalPixels(out width, out height);
            HPS.OffScreenWindowKey offscreenWindowKey = HPS.Database.CreateOffScreenWindow(width, height, oswok);
            offscreenWindowKey.SetCamera(cameraKit);
            offscreenWindowKey.SetVisibility(visibilityKit);
            offscreenWindowKey.IncludeSegment(modelKey);
            HPS.UpdateNotifier updateNotifier = offscreenWindowKey.UpdateWithNotifier();
            updateNotifier.Wait();

            /* This block of code shows how to output the image rendered to the off-screen window to a file
	         * 1. Obtain an imageKit from the off-screen window's WindowOptionControl
	         * 2. Obtain the image data as an array of bytes using the ImageKit.ShowData() method
	         * 3. Output the data to a file in binary format */
            HPS.ImageKit imageKit;
            byte[] imageData;
            bool exportSuccessful = true;
            string file_out = OUTPUT_FOLDER + "/image_output.jpeg";

            /*The next block of code shows you how to create a TextKit for later usage
             * 1. Create a TextKit
             * 2. Set the textkit's formatting options such as position, region, background, and alignment
             * 3. Use the GetMaterialMappingControl to define the background's color properties
             * 4. Leave the TextKit uninserted to facilitate the error and success massage to be defined later */

            HPS.SegmentKey results_segment = modelKey.Subsegment();

            HPS.TextKit textKit = new HPS.TextKit();
            textKit.SetSize(0.02f, Text.SizeUnits.SubscreenRelative);
            HPS.Point[] point = { new HPS.Point(-0.95f, -0.70f, 10), new HPS.Point(0.95f, -0.70f, 10), new HPS.Point(0, -1, 10) };
            textKit.SetRegion(point, Text.RegionAlignment.Center, Text.RegionFitting.Left, false, false, true);
            textKit.SetBackground(true, "rounded box");
            textKit.SetAlignment(HPS.Text.Alignment.BottomLeft);
            results_segment.GetMaterialMappingControl().SetFaceColor(new RGBAColor(1, 1, 1));
            results_segment.GetMaterialMappingControl().SetEdgeColor(new RGBAColor(0, 0, 0));
            results_segment.GetVisibilityControl().SetEdges(true);


            offscreenWindowKey.GetWindowOptionsControl().ShowImage(HPS.Image.Format.Jpeg, out imageKit);
            imageKit.ShowData(out imageData);
            try
            {
                using (System.IO.BinaryWriter writer = new System.IO.BinaryWriter(System.IO.File.OpenWrite(file_out)))
                {
                    writer.Write(imageData);
                    writer.Close();
                }
            }
            catch (System.Exception)
            {
                exportSuccessful = false;
            }

            if (!exportSuccessful)
            {
                string error_message;
                error_message = "Image Export Unsuccessful";
                textKit.SetText(error_message);
                modelKey.InsertText(textKit);
                view.Update();
            }
            else
            {
                string success_message = "Image Export completed successfully\n" +
                    "You can find the PNG image here:\n" +
                    OUTPUT_FOLDER + "/image_output.jpeg";

                int index = success_message.IndexOf(@"/");
                success_message = success_message.Substring(0, index) + "\n    " + success_message.Substring(index, success_message.Length - index);

                textKit.SetText(success_message);
                results_segment.InsertText(textKit);
                view.Update();
            }
        }
    }
}
